"use strict";
exports.__esModule = true;
exports.toSfen = exports.fromSfen = exports.toCSA = exports.fromPreset = void 0;
var Color_1 = require("./Color");
var presets_1 = require("./presets");
var Piece_1 = require("./Piece");
function fromPreset(shogi, setting) {
    var board = [];
    var hands = [[], []];
    var turn;
    if (setting.preset !== "OTHER") {
        var initial = (0, presets_1.getInitialFromPreset)(setting.preset);
        for (var i = 0; i < 9; i++) {
            board[i] = [];
            for (var j = 0; j < 9; j++) {
                var csa = initial.board[j].slice(24 - i * 3, 24 - i * 3 + 3);
                board[i][j] = csa === " * " ? null : new Piece_1["default"](csa);
            }
        }
        turn = initial.turn;
    }
    else {
        for (var i = 0; i < 9; i++) {
            board[i] = [];
            for (var j = 0; j < 9; j++) {
                var p = setting.data.board[i][j];
                board[i][j] = p.kind
                    ? new Piece_1["default"]((p.color === Color_1["default"].Black ? "+" : "-") + p.kind)
                    : null;
            }
        }
        turn = setting.data.color;
        for (var c = 0; c < 2; c++) {
            for (var k in setting.data.hands[c]) {
                // eslint-disable-next-line no-prototype-builtins
                if (setting.data.hands[c].hasOwnProperty(k)) {
                    var csa = (c === 0 ? "+" : "-") + k;
                    for (var i = 0; i < setting.data.hands[c][k]; i++) {
                        hands[c].push(new Piece_1["default"](csa));
                    }
                }
            }
        }
    }
    shogi.board = board;
    shogi.turn = turn;
    shogi.hands = hands;
}
exports.fromPreset = fromPreset;
function toCSA(shogi) {
    var ret = [];
    for (var y = 0; y < 9; y++) {
        var line = "P" + (y + 1);
        for (var x = 8; x >= 0; x--) {
            var piece = shogi.board[x][y];
            line += piece == null ? " * " : piece.toCSAString();
        }
        ret.push(line);
    }
    for (var i = 0; i < 2; i++) {
        var line = "P" + "+-"[i];
        for (var _i = 0, _a = shogi.hands[i]; _i < _a.length; _i++) {
            var hand = _a[_i];
            line += "00" + hand.kind;
        }
        ret.push(line);
    }
    ret.push(shogi.turn === Color_1["default"].Black ? "+" : "-");
    return ret.join("\n");
}
exports.toCSA = toCSA;
function fromSfen(shogi, sfen) {
    var board = [];
    for (var i = 0; i < 9; i++) {
        board[i] = [];
        for (var j = 0; j < 9; j++) {
            board[i][j] = null;
        }
    }
    var segments = sfen.split(" ");
    if (segments[1] !== "w" && segments[1] !== "b")
        throw new Error("Invalid SFEN");
    var sfenBoard = segments[0];
    var x = 8;
    var y = 0;
    for (var i = 0; i < sfenBoard.length; i++) {
        var c = sfenBoard[i];
        if (c === "+") {
            i++;
            c += sfenBoard[i];
        }
        if (c.match(/^[1-9]$/)) {
            x -= Number(c);
        }
        else if (c === "/") {
            y++;
            x = 8;
        }
        else {
            board[x][y] = Piece_1["default"].fromSFENString(c);
            x--;
        }
    }
    shogi.board = board;
    shogi.turn = segments[1] === "b" ? Color_1["default"].Black : Color_1["default"].White;
    var hands = [[], []];
    var sfenHands = segments[2];
    if (sfenHands !== "-") {
        while (sfenHands.length > 0) {
            var count = 1;
            var m = sfenHands.match(/^[0-9]+/);
            if (m) {
                count = Number(m[0]);
                sfenHands = sfenHands.slice(m[0].length);
            }
            for (var i = 0; i < count; i++) {
                var piece = Piece_1["default"].fromSFENString(sfenHands[0]);
                hands[piece.color].push(piece);
            }
            sfenHands = sfenHands.slice(1);
        }
    }
    shogi.hands = hands;
}
exports.fromSfen = fromSfen;
function toSfen(shogi, moveCount) {
    var ret = [];
    var sfenBoard = [];
    for (var y = 0; y < 9; y++) {
        var line = "";
        var empty = 0;
        for (var x = 8; x >= 0; x--) {
            var piece = shogi.board[x][y];
            if (piece == null) {
                empty++;
            }
            else {
                if (empty > 0) {
                    line += "" + empty;
                    empty = 0;
                }
                line += piece.toSFENString();
            }
        }
        if (empty > 0) {
            line += "" + empty;
        }
        sfenBoard.push(line);
    }
    ret.push(sfenBoard.join("/"));
    ret.push(shogi.turn === Color_1["default"].Black ? "b" : "w");
    if (shogi.hands[0].length === 0 && shogi.hands[1].length === 0) {
        ret.push("-");
    }
    else {
        var sfenHands = "";
        var kinds = ["R", "B", "G", "S", "N", "L", "P", "r", "b", "g", "s", "n", "l", "p"];
        var count = {};
        for (var i = 0; i < 2; i++) {
            for (var _i = 0, _a = shogi.hands[i]; _i < _a.length; _i++) {
                var hand = _a[_i];
                var key = hand.toSFENString();
                count[key] = (count[key] || 0) + 1;
            }
        }
        for (var _b = 0, kinds_1 = kinds; _b < kinds_1.length; _b++) {
            var kind = kinds_1[_b];
            if (count[kind] > 0) {
                sfenHands += (count[kind] > 1 ? count[kind] : "") + kind;
            }
        }
        ret.push(sfenHands);
    }
    ret.push("" + moveCount);
    return ret.join(" ");
}
exports.toSfen = toSfen;
